@@ -1,4 +1,4 @@
-/* $Id: C_FORTRAN_interface.c 313 2015-09-16 20:20:04Z mmaechler $
+/* $Id: C_FORTRAN_interface.c 357 2020-02-07 14:14:01Z thothorn $
 *
 *  wrapper for calling R's random number generator from
 *  the original FORTRAN code
@@ -14,3 +14,11 @@ double F77_SUB(sqrtqchisqint)(int *n, double *p) {
 double F77_SUB(phid)(double *x){ return pnorm(*x, 0.0, 1.0, 1, 0); }
 double F77_SUB(studnt)(int *nu, double *x){ return pt(x[0], (double) nu[0], 1, 0); }
 
+double F77_SUB(mvphi)(double const *z){
+  return pnorm5(*z, 0., 1., 1L, 0L);
+}
+
+double F77_SUB(mvphnv)(double const *p){
+  return qnorm5(*p, 0., 1., 1L, 0L);
+}
+
